@@ -1,21 +1,36 @@
-/*       +------------------------------------+
- *       | Inspire Internet Relay Chat Daemon |
- *       +------------------------------------+
+/*
+ * InspIRCd -- Internet Relay Chat Daemon
  *
- *  InspIRCd: (C) 2002-2008 InspIRCd Development Team
- * See: http://www.inspircd.org/wiki/index.php/Credits
+ *   Copyright (C) 2011 Adam <Adam@anope.org>
+ *   Copyright (C) 2009 Daniel De Graaf <danieldg@inspircd.org>
+ *   Copyright (C) 2007, 2009 Dennis Friis <peavey@inspircd.org>
+ *   Copyright (C) 2005-2009 Craig Edwards <craigedwards@brainbox.cc>
+ *   Copyright (C) 2007-2008 Robin Burchell <robin+git@viroteck.net>
+ *   Copyright (C) 2008 Pippijn van Steenhoven <pip88nl@gmail.com>
  *
- * This program is free but copyrighted software; see
- *	    the file COPYING for details.
+ * This file is part of InspIRCd.  InspIRCd is free software: you can
+ * redistribute it and/or modify it under the terms of the GNU General Public
+ * License as published by the Free Software Foundation, version 2.
  *
- * ---------------------------------------------------
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+ * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ * details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef _HASHCOMP_H_
-#define _HASHCOMP_H_
 
-#include "inspircd_config.h"
-#include "socket.h"
+#ifndef HASHCOMP_H
+#define HASHCOMP_H
+
+#include <cstring>
+#include <string>
+#include <vector>
+#include <deque>
+#include <map>
+#include <set>
 #include "hash_map.h"
 
 /*******************************************************
@@ -33,35 +48,51 @@
  * aware of irc::string.
  *******************************************************/
 
-/** Required namespaces and symbols */
-using namespace std;
+/** Seperate from the other casemap tables so that code *can* still exclusively rely on RFC casemapping
+ * if it must.
+ *
+ * This is provided as a pointer so that modules can change it to their custom mapping tables,
+ * e.g. for national character support.
+ */
+CoreExport extern unsigned const char *national_case_insensitive_map;
 
-/** aton() */
-using irc::sockets::insp_aton;
-
-/** nota() */
-using irc::sockets::insp_ntoa;
-
-#ifndef LOWERMAP
-#define LOWERMAP
 /** A mapping of uppercase to lowercase, including scandinavian
  * 'oddities' as specified by RFC1459, e.g. { -> [, and | -> \
  */
-unsigned const char lowermap[256] = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19,				/* 0-19 */
-				20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39,				/* 20-39 */
-				40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59,				/* 40-59 */
-				60, 61, 62, 63, 64, 97, 98, 99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111,		/* 60-79 */
-				112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 123, 124, 125, 94, 95, 96, 97, 98, 99,		/* 80-99 */
-				100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111, 112, 113, 114, 115, 116, 117, 118, 119,	/* 100-119 */
-				120, 121, 122, 123, 124, 125, 126, 127, 128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139,	/* 120-139 */
-				140, 141, 142, 143, 144, 145, 146, 147, 148, 149, 150, 151, 152, 153, 154, 155, 156, 157, 158, 159,	/* 140-159 */
-				160, 161, 162, 163, 164, 165, 166, 167, 168, 169, 170, 171, 172, 173, 174, 175, 176, 177, 178, 179,	/* 160-179 */
-				180, 181, 182, 183, 184, 185, 186, 187, 188, 189, 190, 191, 192, 193, 194, 195, 196, 197, 198, 199,	/* 180-199 */
-				200, 201, 202, 203, 204, 205, 206, 207, 208, 209, 210, 211, 212, 213, 214, 215, 216, 217, 218, 219,	/* 200-219 */
-				220, 221, 222, 223, 224, 225, 226, 227, 228, 229, 230, 231, 232, 233, 234, 235, 236, 237, 238, 239,	/* 220-239 */
-				240, 241, 242, 243, 244, 245, 246, 247, 248, 249, 250, 251, 252, 253, 254, 255				/* 240-255 */
-};
-#endif
+CoreExport extern unsigned const char rfc_case_insensitive_map[256];
+
+/** Case insensitive map, ASCII rules.
+ * That is;
+ * [ != {, but A == a.
+ */
+CoreExport extern unsigned const char ascii_case_insensitive_map[256];
+
+/** Case sensitive (identity) map.
+ */
+CoreExport extern unsigned const char rfc_case_sensitive_map[256];
+
+template<typename T> const T& SearchAndReplace(T& text, const T& pattern, const T& replace)
+{
+        T replacement;
+        if ((!pattern.empty()) && (!text.empty()))
+        {
+                for (std::string::size_type n = 0; n != text.length(); ++n)
+                {
+                        if (text.length() >= pattern.length() && text.substr(n, pattern.length()) == pattern)
+                        {
+                                /* Found the pattern in the text, replace it, and advance */
+                                replacement.append(replace);
+                                n = n + pattern.length() - 1;
+                        }
+                        else
+                        {
+                                replacement += text[n];
+                        }
+                }
+        }
+        text = replacement;
+        return text;
+}
 
 /** The irc namespace contains a number of helper classes.
  */
@@ -72,7 +103,7 @@ namespace irc
 	 * Case sensitivity is ignored, and the RFC 'character set'
 	 * is adhered to
 	 */
-	struct StrHashComp
+	struct CoreExport StrHashComp
 	{
 		/** The operator () does the actual comparison in hash_map
 		 */
@@ -126,28 +157,31 @@ namespace irc
 
 	/** Compose a hex string from raw data.
 	 * @param raw The raw data to compose hex from
-	 * @pram rawsz The size of the raw data buffer
+	 * @param rawsz The size of the raw data buffer
 	 * @return The hex string.
 	 */
 	CoreExport std::string hex(const unsigned char *raw, size_t rawsz);
 
 	/** This typedef declares irc::string based upon irc_char_traits.
 	 */
-	typedef basic_string<char, irc_char_traits, allocator<char> > string;
+	typedef std::basic_string<char, irc_char_traits, std::allocator<char> > string;
 
 	/** irc::stringjoiner joins string lists into a string, using
 	 * the given seperator string.
 	 * This class can join a vector of std::string, a deque of
-	 * std::string, or a const char** array, using overloaded
+	 * std::string, or a const char* const* array, using overloaded
 	 * constructors.
 	 */
 	class CoreExport stringjoiner
 	{
 	 private:
+
 		/** Output string
 		 */
 		std::string joined;
+
 	 public:
+
 		/** Join elements of a vector, between (and including) begin and end
 		 * @param seperator The string to seperate values with
 		 * @param sequence One or more items to seperate
@@ -155,6 +189,7 @@ namespace irc
 		 * @param end The ending element in the sequence to be joined
 		 */
 		stringjoiner(const std::string &seperator, const std::vector<std::string> &sequence, int begin, int end);
+
 		/** Join elements of a deque, between (and including) begin and end
 		 * @param seperator The string to seperate values with
 		 * @param sequence One or more items to seperate
@@ -162,13 +197,14 @@ namespace irc
 		 * @param end The ending element in the sequence to be joined
 		 */
 		stringjoiner(const std::string &seperator, const std::deque<std::string> &sequence, int begin, int end);
+
 		/** Join elements of an array of char arrays, between (and including) begin and end
 		 * @param seperator The string to seperate values with
 		 * @param sequence One or more items to seperate
 		 * @param begin The starting element in the sequence to be joined
 		 * @param end The ending element in the sequence to be joined
 		 */
-		stringjoiner(const std::string &seperator, const char** sequence, int begin, int end);
+		stringjoiner(const std::string &seperator, const char* const* sequence, int begin, int end);
 
 		/** Get the joined sequence
 		 * @return A reference to the joined string
@@ -186,17 +222,20 @@ namespace irc
 		/** The mode sequence and its parameters
 		 */
 		std::deque<std::string> sequence;
+
 		/** True if the mode sequence is initially adding
 		 * characters, false if it is initially removing
 		 * them
 		 */
 		bool adding;
 	 public:
+
 		/** Construct a new modestacker.
 		 * @param add True if the stack is adding modes,
 		 * false if it is removing them
 		 */
 		modestacker(bool add);
+
 		/** Push a modeletter and its parameter onto the stack.
 		 * No checking is performed as to if this mode actually
 		 * requires a parameter. If you stack invalid mode
@@ -206,6 +245,7 @@ namespace irc
 		 * @param parameter The parameter for the mode
 		 */
 		void Push(char modeletter, const std::string &parameter);
+
 		/** Push a modeletter without parameter onto the stack.
 		 * No checking is performed as to if this mode actually
 		 * requires a parameter. If you stack invalid mode
@@ -214,12 +254,15 @@ namespace irc
 		 * @param modeletter The mode letter to insert
 		 */
 		void Push(char modeletter);
+
 		/** Push a '+' symbol onto the stack.
 		 */
 		void PushPlus();
+
 		/** Push a '-' symbol onto the stack.
 		 */
 		void PushMinus();
+
 		/** Return zero or more elements which form the
 		 * mode line. This will be clamped to a max of
 		 * MAXMODES items (MAXMODES-1 mode parameters and
@@ -227,7 +270,7 @@ namespace irc
 		 * characters. As specified below, this function
 		 * should be called in a loop until it returns zero,
 		 * indicating there are no more modes to return.
-		 * @param result The deque to populate. This will
+		 * @param result The vector to populate. This will not
 		 * be cleared before it is used.
 		 * @param max_line_size The maximum size of the line
 		 * to build, in characters, seperate to MAXMODES.
@@ -236,7 +279,16 @@ namespace irc
 		 * returns 0, in case there are multiple lines of
 		 * mode changes to be obtained.
 		 */
-		int GetStackedLine(std::deque<std::string> &result, int max_line_size = 360);
+		int GetStackedLine(std::vector<std::string> &result, int max_line_size = 360);
+
+		/** deprecated compatability interface - TODO remove */
+		int GetStackedLine(std::deque<std::string> &result, int max_line_size = 360) {
+			std::vector<std::string> r;
+			int n = GetStackedLine(r, max_line_size);
+			result.clear();
+			result.insert(result.end(), r.begin(), r.end());
+			return n;
+		}
 	};
 
 	/** irc::tokenstream reads a string formatted as per RFC1459 and RFC2812.
@@ -254,22 +306,30 @@ namespace irc
 	class CoreExport tokenstream
 	{
 	 private:
+
 		/** Original string
 		 */
 		std::string tokens;
+
 		/** Last position of a seperator token
 		 */
 		std::string::iterator last_starting_position;
+
 		/** Current string position
 		 */
 		std::string::iterator n;
+
 		/** True if the last value was an ending value
 		 */
 		bool last_pushed;
 	 public:
+
 		/** Create a tokenstream and fill it with the provided data
 		 */
 		tokenstream(const std::string &source);
+
+		/** Destructor
+		 */
 		~tokenstream();
 
 		/** Fetch the next token from the stream as a std::string
@@ -302,7 +362,7 @@ namespace irc
 	 * the next token, until none remain, at which point the method returns
 	 * an empty string.
 	 */
-	class CoreExport sepstream : public classbase
+	class CoreExport sepstream
 	{
 	 private:
 		/** Original string.
@@ -321,6 +381,9 @@ namespace irc
 		/** Create a sepstream and fill it with the provided data
 		 */
 		sepstream(const std::string &source, char seperator);
+
+		/** Destructor
+		 */
 		virtual ~sepstream();
 
 		/** Fetch the next token from the stream
@@ -328,12 +391,12 @@ namespace irc
 		 * @return True if tokens still remain, false if there are none left
 		 */
 		virtual bool GetToken(std::string &token);
-		
+
 		/** Fetch the entire remaining stream, without tokenizing
 		 * @return The remaining part of the stream
 		 */
 		virtual const std::string GetRemaining();
-		
+
 		/** Returns true if the end of the stream has been reached
 		 * @return True if the end of the stream has been reached, otherwise false
 		 */
@@ -372,188 +435,66 @@ namespace irc
 	 * start or end < 0) then GetToken() will return the first element
 	 * of the pair of numbers.
 	 */
-	class CoreExport portparser : public classbase
+	class CoreExport portparser
 	{
 	 private:
+
 		/** Used to split on commas
 		 */
-		commasepstream* sep;
+		commasepstream sep;
+
 		/** Current position in a range of ports
 		 */
 		long in_range;
+
 		/** Starting port in a range of ports
 		 */
 		long range_begin;
+
 		/** Ending port in a range of ports
 		 */
 		long range_end;
+
 		/** Allow overlapped port ranges
 		 */
 		bool overlapped;
+
 		/** Used to determine overlapping of ports
 		 * without O(n) algorithm being used
 		 */
-		std::map<long, bool> overlap_set;
+		std::set<long> overlap_set;
+
 		/** Returns true if val overlaps an existing range
 		 */
 		bool Overlaps(long val);
 	 public:
+
 		/** Create a portparser and fill it with the provided data
 		 * @param source The source text to parse from
 		 * @param allow_overlapped Allow overlapped ranges
 		 */
 		portparser(const std::string &source, bool allow_overlapped = true);
-		/** Frees the internal commasepstream object
-		 */
-		~portparser();
+
 		/** Fetch the next token from the stream
 		 * @return The next port number is returned, or 0 if none remain
 		 */
 		long GetToken();
 	};
 
-	/** Used to hold a bitfield definition in dynamicbitmask.
-	 * You must be allocated one of these by dynamicbitmask::Allocate(),
-	 * you should not fill the values yourself!
-	 */
-	typedef std::pair<size_t, unsigned char> bitfield;
-
-	/** The irc::dynamicbitmask class is used to maintain a bitmap of
-	 * boolean values, which can grow to any reasonable size no matter
-	 * how many bitfields are in it.
-	 *
-	 * It starts off at 32 bits in size, large enough to hold 32 boolean
-	 * values, with a memory allocation of 8 bytes. If you allocate more
-	 * than 32 bits, the class will grow the bitmap by 8 bytes at a time
-	 * for each set of 8 bitfields you allocate with the Allocate()
-	 * method.
-	 *
-	 * This method is designed so that multiple modules can be allocated
-	 * bit values in a bitmap dynamically, rather than having to define
-	 * costs in a fixed size unsigned integer and having the possibility
-	 * of collisions of values in different third party modules.
-	 *
-	 * IMPORTANT NOTE:
-	 *
-	 * To use this class, you must derive from it.
-	 * This is because each derived instance has its own freebits array
-	 * which can determine what bitfields are allocated on a TYPE BY TYPE
-	 * basis, e.g. an irc::dynamicbitmask type for userrecs, and one for
-	 * chanrecs, etc. You should inheret it in a very simple way as follows.
-	 * The base class will resize and maintain freebits as required, you are
-	 * just required to make the pointer static and specific to this class
-	 * type.
-	 *
-	 * \code
-	 * class mydbitmask : public irc::dynamicbitmask
-	 * {
-	 *  private:
-	 *
-	 *      static unsigned char* freebits;
-	 *
-	 *  public:
-	 *
-	 *      mydbitmask() : irc::dynamicbitmask()
-	 *      {
-	 *	  freebits = new unsigned char[this->bits_size];
-	 *	  memset(freebits, 0, this->bits_size);
-	 *      }
-	 *
-	 *      ~mydbitmask()
-	 *      {
-	 *	  delete[] freebits;
-	 *      }
-	 *
-	 *      unsigned char* GetFreeBits()
-	 *      {
-	 *	  return freebits;
-	 *      }
-	 *
-	 *      void SetFreeBits(unsigned char* freebt)
-	 *      {
-	 *	  freebits = freebt;
-	 *      }
-	 * };
-	 * \endcode
-	 */
-	class CoreExport dynamicbitmask : public classbase
-	{
-	 private:
-		/** Data bits. We start with four of these,
-		 * and we grow the bitfield as we allocate
-		 * more than 32 entries with Allocate().
-		 */
-		unsigned char* bits;
-	 protected:
-		/** Current set size (size of freebits and bits).
-		 * Both freebits and bits will ALWAYS be the
-		 * same length.
-		 */
-		unsigned char bits_size;
-	 public:
-		/** Allocate the initial memory for bits and
-		 * freebits and zero the memory.
-		 */
-		dynamicbitmask();
-
-		/** Free the memory used by bits and freebits
-		 */
-		virtual ~dynamicbitmask();
-
-		/** Allocate an irc::bitfield.
-		 * @return An irc::bitfield which can be used
-		 * with Get, Deallocate and Toggle methods.
-		 * @throw Can throw std::bad_alloc if there is
-		 * no ram left to grow the bitmask.
-		 */
-		bitfield Allocate();
-
-		/** Deallocate an irc::bitfield.
-		 * @param An irc::bitfield to deallocate.
-		 * @return True if the bitfield could be
-		 * deallocated, false if it could not.
-		 */
-		bool Deallocate(bitfield &pos);
-
-		/** Toggle the value of a bitfield.
-		 * @param pos A bitfield to allocate, previously
-		 * allocated by dyamicbitmask::Allocate().
-		 * @param state The state to set the field to.
-		 */
-		void Toggle(bitfield &pos, bool state);
-
-		/** Get the value of a bitfield.
-		 * @param pos A bitfield to retrieve, previously
-		 * allocated by dyamicbitmask::Allocate().
-		 * @return The value of the bitfield.
-		 * @throw Will throw ModuleException if the bitfield
-		 * you provide is outside of the range
-		 * 0 >= bitfield.first < size_bits.
-		 */
-		bool Get(bitfield &pos);
-
-		/** Return the size in bytes allocated to the bits
-		 * array.
-		 * Note that the actual allocation is twice this,
-		 * as there are an equal number of bytes allocated
-		 * for the freebits array.
-		 */
-		unsigned char GetSize();
-
-		/** Get free bits mask
-		 */
-		virtual unsigned char* GetFreeBits() { return NULL; }
-
-		/** Set free bits mask
-		 */
-		virtual void SetFreeBits(unsigned char* freebits) { }
-	};
-
 	/** Turn _ characters in a string into spaces
 	 * @param n String to translate
 	 * @return The new value with _ translated to space.
 	 */
 	CoreExport const char* Spacify(const char* n);
+
+	struct hash
+	{
+		/** Hash an irc::string using RFC1459 case sensitivity rules
+		 * @param s A string to hash
+		 * @return The hash value
+		 */
+		size_t CoreExport operator()(const irc::string &s) const;
+	};
 }
 
 /* Define operators for using >> and << with irc::string to an ostream on an istream. */
@@ -614,6 +555,20 @@ inline bool operator== (const irc::string& leftval, const std::string& rightval)
 	return (leftval == rightval.c_str());
 }
 
+/* Define operators != for irc::string to std::string for easy comparison
+ */
+inline bool operator!= (const irc::string& leftval, const std::string& rightval)
+{
+	return !(leftval == rightval.c_str());
+}
+
+/* Define operators != for std::string to irc::string for easy comparison
+ */
+inline bool operator!= (const std::string& leftval, const irc::string& rightval)
+{
+	return !(leftval.c_str() == rightval);
+}
+
 /** Assign an irc::string to a std::string.
  */
 inline std::string assign(const irc::string &other) { return other.c_str(); }
@@ -637,13 +592,13 @@ inline std::string& trim(std::string &str)
 }
 
 /** Hashing stuff is totally different on vc++'s hash_map implementation, so to save a buttload of
- * #ifdefs we'll just do it all at once
+ * \#ifdefs we'll just do it all at once. Except, of course, with TR1, when it's the same as GCC.
  */
-namespace nspace
-{
+BEGIN_HASHMAP_NAMESPACE
+
 	/** Hashing function to hash irc::string
 	 */
-#ifdef WINDOWS
+#if defined(_WIN32) && !defined(HAS_TR1_UNORDERED)
 	template<> class CoreExport hash_compare<irc::string, std::less<irc::string> >
 	{
 	public:
@@ -654,9 +609,9 @@ namespace nspace
 		bool operator()(const irc::string & s1, const irc::string & s2) const
 		{
 			if(s1.length() != s2.length()) return true;
-			return (irc::irc_char_traits::compare(s1.c_str(), s2.c_str(), s1.length()) < 0);
+			return (irc::irc_char_traits::compare(s1.c_str(), s2.c_str(), (size_t)s1.length()) < 0);
 		}
-		
+
 		/** Hash an irc::string value for hash_map
 		 */
 		size_t operator()(const irc::string & s) const;
@@ -672,9 +627,9 @@ namespace nspace
 		bool operator()(const std::string & s1, const std::string & s2) const
 		{
 			if(s1.length() != s2.length()) return true;
-			return (irc::irc_char_traits::compare(s1.c_str(), s2.c_str(), s1.length()) < 0);
+			return (irc::irc_char_traits::compare(s1.c_str(), s2.c_str(), (size_t)s1.length()) < 0);
 		}
-		
+
 		/** Hash a std::string using RFC1459 case sensitivity rules
 		* @param s A string to hash
 		* @return The hash value
@@ -682,30 +637,25 @@ namespace nspace
 		size_t operator()(const std::string & s) const;
 	};
 #else
-	template<> struct hash<irc::string>
+
+	/* XXX FIXME: Implement a hash function overriding std::string's that works with TR1! */
+
+#ifdef HASHMAP_DEPRECATED
+	struct insensitive
+#else
+	CoreExport template<> struct hash<std::string>
+#endif
 	{
-		/** Hash an irc::string using RFC1459 case sensitivity rules
-		 * @param s A string to hash
-		 * @return The hash value
-		 */
-		size_t operator()(const irc::string &s) const;
+		size_t CoreExport operator()(const std::string &s) const;
 	};
 
-	template<> struct hash<std::string>
-	{
-		/** Hash a std::string using RFC1459 case sensitivity rules
-		* @param s A string to hash
-		* @return The hash value
-		*/
-		size_t operator()(const string &s) const;
-	};
 #endif
 
 	/** Convert a string to lower case respecting RFC1459
 	* @param n A string to lowercase
 	*/
 	void strlower(char *n);
-}
+
+END_HASHMAP_NAMESPACE
 
 #endif
-
