@@ -202,10 +202,18 @@ void Channel::SetDefaultModes()
 		if (mode)
 		{
 			if (mode->GetNumParams(true))
+			{
 				list.GetToken(parameter);
+				// If the parameter begins with a ':' then it's invalid
+				if (parameter.c_str()[0] == ':')
+					continue;
+			}
 			else
 				parameter.clear();
 
+			if ((mode->GetNumParams(true)) && (parameter.empty()))
+				continue;
+
 			mode->OnModeChange(ServerInstance->FakeClient, ServerInstance->FakeClient, this, parameter, true);
 		}
 	}
