@@ -390,8 +390,17 @@ bool ParseStack::ParseExec(const std::string& name, int flags, const std::string
 
 bool ConfigTag::readString(const std::string& key, std::string& value, bool allow_lf)
 {
+#ifdef __clang__
+# pragma clang diagnostic push
+# pragma clang diagnostic ignored "-Wunknown-pragmas"
+# pragma clang diagnostic ignored "-Wundefined-bool-conversion"
+#endif
+	// TODO: this is undefined behaviour but changing the API is too risky for 2.0.
 	if (!this)
 		return false;
+#ifdef __clang__
+# pragma clang diagnostic pop
+#endif
 	for(std::vector<KeyVal>::iterator j = items.begin(); j != items.end(); ++j)
 	{
 		if(j->first != key)
