@@ -163,9 +163,6 @@ class CommandWatch : public Command
 			/* Yup, is on my list */
 			watchlist::iterator n = wl->find(nick);
 
-			if (!wl)
-				return CMD_FAILURE;
-
 			if (n != wl->end())
 			{
 				if (!n->second.empty())
@@ -420,7 +417,7 @@ class Modulewatch : public Module
 		{
 			for (std::deque<User*>::iterator n = x->second.begin(); n != x->second.end(); n++)
 			{
-				(*n)->WriteNumeric(inum, numeric);
+				(*n)->WriteNumeric(inum, (*n)->nick + " " + numeric);
 			}
 		}
 
